@@ -1,5 +1,5 @@
 `getAllTerms.default` <-
-function(x, ...) getAllTerms(as.formula(formula(x)), ...)
+function(x, ...) getAllTerms.formula(as.formula(formula(x)), ...)
 
 `getAllTerms.terms` <-
 function(x, offset = TRUE, ...) {
@@ -73,7 +73,7 @@ function(x, ...) {
 `getAllTerms.glmer` <- # For backwards compatibility
 `getAllTerms.lmer` <-  # with older versions of lme4
 `getAllTerms.mer` <-
-function(x, ...) getAllTerms(formula(x), ...)
+function(x, ...) getAllTerms(lme4::formula(x), ...)
 
 # Apparently there is no (explicit) intercept in coxph, but 'terms' gives
 # attr(,"intercept") == 1.
@@ -83,6 +83,5 @@ function(x, ...) getAllTerms(formula(x), ...)
 	return(ret)
 }
 
-
 `getAllTerms` <-
 function(x, ...) UseMethod("getAllTerms")
